/**
 *  qdsock.h by Tyler Montbriand, 2004
 *  Free software as in free beer.  Do whatever you want with it..
 */

/***********************************************************************
   qdtcp is a quick-and-dirty TCP socket class.  Thanks to SDL_net,
   it seems to work quite well despite being thrown together in 5 
   minutes.
 ***********************************************************************/

#ifndef __QDSOCK_H__
#define __QDSOCK_H__

#ifndef __cplusplus
#error "qdsock.h needs C++ compilation"
#endif

#include <SDL/SDL_net.h>

  // Needed for memcpy
#include <string.h>
#include "qdwrap.h"

#define UDP_MAX_SIZE 1472

/**
 * A simple wrapper class for IPaddress.  Has nice default operators
 * to resolve host addresses.  Comparable, can be used in sorted lists.
 *
 * example:
 *  {
 *    qdip ip("some.address.net",8888);
 *    fprintf(stderr,"Resolved to 0x%08x\n",ip.host);
 *  }
 */
class qdip:public IPaddress
{
public:
  inline qdip() { host=INADDR_NONE; port=0; }

  inline qdip(const IPaddress &ip)
  { host=ip.host;  port=ip.port; }

  inline qdip(const char *h, Uint16 p)
  { resolve(h,p); }

  inline void resolve(const char *h, Uint16 p)
  { SDLNet_ResolveHost(this,(char *)h,p); }

    // Ordering is not guaranteed, but uniqueness is.
  inline bool operator <(const IPaddress &ip) const
  { return(order(*this)<order(ip)); }

  inline bool operator ==(const IPaddress &ip) const
  { return((host==ip.host)&&(port==ip.port)); }

  inline void operator=(const IPaddress &ip) { host=ip.host; port=ip.port; }

  inline bool Good() { return(host!=INADDR_NONE); }

private:
  inline Uint64 order(const IPaddress &ip) const
  {
    Uint64 v=ip.host;
    v<<=32;
    v|=ip.port;
    return(v);
  }
};

/**
 * quick-and-dirty wrapper class for SDLNet TCP sockets.  Provides
 * really useful default constructors and destructors -- opens
 * sockets when it goes into scope and closes them when it goes out of 
 * scope.
 * Example:
 *
 *  #include <string.h>
 *  #include "qdsock.h"
 *  #include "qdnet.h"
 *  int main()
 *  {
 *    char *msg="You farted!"; // String to send
 *    qdnet net;               // Initializes SDL and SDL_net
 *    if(!net.Good())          // Quit on error
 *      return(1);
 *    {
 *      qdtcp sock("some.address.net",9999); // Open socket
 *      if(sock.Good())
 *        SDLNet_TCP_Send(sock,msg,strlen(msg)+1); // Share this wisdom
 *      else
 *        fprintf(stderr,"Couldn't open socket\n");
 *    } // socket goes out of scope here, automatically closed
 *  } // qdnet goes out of scope here, automatically quits SDL and SDL_net
 */
class qdtcp:public qdwrap<struct _TCPsocket>
{
public:
  inline qdtcp():qdwrap<struct _TCPsocket>(NULL) { }
  inline qdtcp(TCPsocket sockin):qdwrap<struct _TCPsocket>(sockin) { }
  inline qdtcp(char *host, Uint16 port):qdwrap<struct _TCPsocket>(NULL)
  { open(host,port); }

  inline ~qdtcp() { close(); }

  inline void operator=(TCPsocket sock)
  {
    close();
    val=sock;
  }

  inline bool Activity() { return(SDLNet_SocketReady(val)!=0); }

  inline TCPsocket accept() { return(SDLNet_TCP_Accept(val)); }

  inline bool open(char *host, Uint16 port)
  {
    qdip ip(host,port);
    return(open(&ip));
  }

  inline bool open(IPaddress *ip)
  {
    close();
    val=SDLNet_TCP_Open(ip);
    return(val!=NULL);
  }

  inline void close()
  {
    if(val!=NULL)
    {
      SDLNet_TCP_Close(val);
      val=NULL;
    }
  }
};

/**
 * Really simple UDP socket wrapper class.  Constructor opens sockets,
 * destructor closes sockets.  Really handy.
 * Example:
 *
 *  #include <string.h>
 *  #include "qdsock.h"
 *  #include "qdnet.h"
 *  int main()
 *  {
 *    char *msg="You farted!"; // String to send
 *    qdnet net;               // Initializes SDL and SDL_net
 *    if(!net.Good())          // Quit on error
 *      return(1);
 *    {
 *      qdudp    sock(0); // Open anonymous socket
 *      qdip     ip("some.address.net",9999);
 *      qdpacket packet(ip,msg,sizeof(msg)+1);
 *
 *      if(sock.Good())
 *        SDLNet_UDP_Send(sock,-1,&packet); // Share this wisdom
 *      else
 *        fprintf(stderr,"Couldn't open socket\n");
 *    } // socket goes out of scope here, automatically closed
 *  } // qdnet goes out of scope here, automatically quits SDL and SDL_net
 */
class qdudp:public qdwrap<struct _UDPsocket>
{
public:
  qdudp():qdwrap<struct _UDPsocket>(NULL) { }
  qdudp(Uint16 port):qdwrap<struct _UDPsocket>(NULL) { open(port); }

  inline bool open(Uint16 port)
  {
    close();

    val=SDLNet_UDP_Open(port);
    return(val!=NULL);
  }

  inline bool Activity() { return(SDLNet_SocketReady(val)!=0); }

  void close()
  {
    if(val!=NULL)
    {
      SDLNet_UDP_Close(val);
      val=NULL;
    }
  }

  inline qdip *Peer(int chan=-1)
  {
    if(val==NULL)
      return(NULL);
    return((qdip *)SDLNet_UDP_GetPeerAddress(val,chan));
  }

  ~qdudp() { close(); }
};

/**
 * Quick-and-dirty packet class.  Default constructor sets sane default
 * values.  Can be cast to directly from a UDPpacket *, just don't use 
 * the constructors/destructors -- they modify pointers.
 *
 * See the UDPsocket example for usage.
 */
class qdpacket:public UDPpacket
{
public:
  qdpacket(IPaddress ip, void *dat, Uint32 size)
  {
    data=new Uint8[UDP_MAX_SIZE];
    maxlen=UDP_MAX_SIZE;
    len=0;
    channel=0;
    status=0;
    address=ip;
    set(dat,size);
  }

  qdpacket(Uint16 size=UDP_MAX_SIZE)
  {
    if(size>UDP_MAX_SIZE) size=UDP_MAX_SIZE;
    data=new Uint8[size];
    maxlen=size;
    len=0;
    channel=0;
    status=0;
    address.host=INADDR_NONE;
    address.port=0;
  }

  ~qdpacket()
  { if(data!=NULL) { delete []data; } }

  bool set(const void *d, Uint32 size)
  {
    if((size>maxlen)||(data==NULL)||(this==NULL))
      return(false);

    memcpy(data,d,size);
    len=size;
    return(true);
  }
};

class qdsocketset
{
public:
  inline qdsocketset(int max=16) { set=SDLNet_AllocSocketSet(max); }
  inline ~qdsocketset() { if(set) { SDLNet_FreeSocketSet(set); set=NULL; } }
  inline bool Good() { return(set!=NULL); }
private:
  SDLNet_SocketSet set;
};

#endif/*__QDSOCK_H__*/
