/**
 *  qdtime.h by Tyler Montbriand, 2004
 *  Free software as in free beer.  Do whatever you want with it..
 */
#ifndef __QDTIME_H__
#define __QDTIME_H__

#ifndef __cplusplus
#error "qdtime.h needs C++ compilation"
#endif

#include <SDL/SDL_types.h>
#include <SDL/SDL_timer.h>
/**
 * Quick-and-dirty SDL stopwatch class.  Tells how many milliseconds 
 * have gone by since the timer has last been reset.
 * Example:
 *
 *  #include <stdio.h>
 *  #include "qdsdl.h"
 *  #include "qdtime.h"
 *
 *  int main()
 *  {
 *    qdsdl sdl(SDL_INIT_TIMER);
 *    if(!sdl.Good())
 *      return(1);
 *
 *    {
 *      stopwatch t;
 *      SDL_Delay(1000); // Sleep for approximately 1000 milliseconds
 *      fprintf(stderr,"Timer says %d milliseconds have passed\n",t.time());
 *    }
 *  }
 */
class stopwatch
{
public:
  inline    stopwatch()                 { reset();                     }
    // Resets the timer to zero.
  inline void   reset()                 { tick=SDL_GetTicks();         }
    // Returns the time in milliseconds since the last reset.
  inline Sint32  time()                 { return(SDL_GetTicks()-tick); }
    // Advances the timer.
  inline void operator +=(Sint32 ticks) { tick+=ticks; }
    // Moves the timer backwards a bit.
  inline void operator -=(Sint32 ticks) { tick-=ticks; }
private:
  Sint32 tick;
};

/**
 * Quick-and-dirty timer class.  Destructor stops the timer.
 * Example:
 *
 *  #include <stdio.h>
 *  #include "qdsdl.h"
 *  #include "qdtime.h"
 *
 *  Uint32 timefunc(Uint32 i, void *dat)
 *  {
 *    printf("Survey Says:  %s\n",dat);
 *    return(i);
 *  }
 *
 *  int main()
 *  {
 *    qdsdl sdl(SDL_INIT_TIMER); // Needs SDL_INIT_TIMER
 *    if(!sdl.Good())
 *      return(1);
 *
 *    {
 *      qdtimer timer(200,tinefunc,"You Farted!");
 *      SDL_Delay(2000); // Wait 2 seconds, timer should keep running
 *    } // Timer goes out of scope and stops
 *    fprintf(stderr,"Done.\n");
 *    SDL_Delay(2000); // Wait another 2 seconds to prove it stopped
 *    fprintf(stderr,"Really done.\n");
 *  } // sdl goes out of scope and quits SDL
 */
class qdtimer
{
public:
    // Complex constructor, creates the timer on instantiation
  inline qdtimer(Uint32 interval, SDL_NewTimerCallback c, void *p=NULL)
    :t(NULL)
  { start(interval,c,p); }

    // Simple constructor, no timer is created
  inline qdtimer():t(NULL) { }

    // Starts timer.  Stops currently running timer if any.
  inline bool start(Uint32 interval, SDL_NewTimerCallback c, void *p)
  {
    stop();
    t=SDL_AddTimer(interval,c,p);
    return(t!=NULL);
  }

    // Stops any running timer.
  inline void stop()
  {
    if(t!=NULL)
    {
      SDL_RemoveTimer(t);
      t=NULL;
    }
  }
    // destructor.  If timer is running, stops it.
  inline ~qdtimer() { stop(); }
private:
  SDL_TimerID t;
};

#endif/*__QDTIME_H__*/
